@@ -1,20 +1,25 @@
-from pathlib import Path
+import json
 import re
 import sys
 import warnings
+from pathlib import Path
 
-import numpy as np 
-
-import json
 import h5py
+import numpy as np
 
-from npyx.utils import assert_int, assert_float
-from npyx.inout import get_binary_file_path, get_npix_sync, chan_map,\
-                       extract_rawChunk, read_metadata, preprocess_binary_file, detect_hardware_filter
+from npyx.gl import check_periods, get_units
+from npyx.inout import (
+    chan_map,
+    detect_hardware_filter,
+    extract_rawChunk,
+    get_binary_file_path,
+    get_npix_sync,
+    preprocess_binary_file,
+    read_metadata,
+)
 from npyx.spk_t import ids, trn, trn_filtered
 from npyx.spk_wvf import wvf_dsmatch
-from npyx.gl import get_units, check_periods
-
+from npyx.utils import assert_float, assert_int
 
 
 # High level C4 functions
@@ -98,7 +103,7 @@ def remove_unit_h5(h5_path, dp, unit, lab_id='hausser'):
 
 
 def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
-                sync_chan_id=None,
+                sync_chan_id=None, overwrite_h5=False,
                 again=False, again_wvf=False, plot_debug=False, verbose=False,
                 dataset=None,
                 raw_window=[0.1, 30.1], center_raw_window_on_spikes=True,
@@ -106,7 +111,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
                 raw_snippet_halfrange=2, mean_wvf_half_range=11,
                 sane_spikes=None, sane_periods=None, sane_before_opto=False, include_fp_fn_mask=True,
                 optostims=None, optostims_from_sync=False, optostims_threshold=None,
-                n_waveforms_for_matching=5000,
+                n_waveforms_for_matching=5000, selective_overwrite=None,
                 **kwargs):
     """
     Add a spike-sorted unit to an HDF5 file (on a phy compatible dataformat).
@@ -172,6 +177,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
                            (handles sync signals without light at beginning of recording)
                            
     - n_waveforms_for_matching: int, number of waveforms to subsample for drift-shift matching
+    - selective_overwrite: list of strings, which fields to specifically recompute even if again is False (much faster if only one field to overwrite).
 
     Additional key-value parameteters:
     - *any_key* = *any_value*
@@ -183,6 +189,9 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
     for this unit.
     """
     
+    if selective_overwrite is not None:
+        assert isinstance(selective_overwrite, list), "Selective_overwrite must be a list of strings."
+    
     dp=Path(dp)
     meta = read_metadata(dp) 
     samp_rate = meta['highpass']['sampling_rate']
@@ -204,7 +213,7 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
         # Define or fetch unit h5 paths
         relative_unit_path, absolute_unit_path = get_unit_paths_h5(h5_file, dataset, unit, lab_id)
         if relative_unit_path in h5_file:
-            if again:
+            if overwrite_h5:
                 del h5_file[relative_unit_path]
                 del h5_file[absolute_unit_path]
             else:
@@ -220,45 +229,29 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             print(f"Adding data at {relative_unit_path} ({absolute_unit_path})...")
 
         # metadata
-        write_to_group(neuron_group, 'lab_id', lab_id, again)
-        write_to_group(neuron_group, 'dataset_id', dataset, again)
-        write_to_group(neuron_group, 'neuron_id', unit, again)
-        write_to_group(neuron_group, 'neuron_absolute_id', neuron_group.name, again)
-        write_to_group(neuron_group, 'sampling_rate', samp_rate, again)
-        write_to_group(neuron_group, 'periods', samp_rate, again)
+        write_to_group(neuron_group, 'lab_id', lab_id, overwrite_h5)
+        write_to_group(neuron_group, 'dataset_id', dataset, overwrite_h5)
+        write_to_group(neuron_group, 'neuron_id', unit, overwrite_h5)
+        write_to_group(neuron_group, 'neuron_absolute_id', neuron_group.name, overwrite_h5)
+        write_to_group(neuron_group, 'sampling_rate', samp_rate, overwrite_h5)
+        write_to_group(neuron_group, 'periods', samp_rate, overwrite_h5)
         
         # add any additional keys passed to this function
         for key, value in kwargs.items():
-            write_to_group(neuron_group, key, value, again)
+            write_to_group(neuron_group, key, value, overwrite_h5)
 
         # spike_times
+        change_spike_train = overwrite_h5 or ("spike_times" in selective_overwrite)
         periods = check_periods(periods)
-        if 'spike_indices' not in neuron_group or again:
+        if 'spike_indices' not in neuron_group or change_spike_train:
             t = trn(dp, unit, periods=periods, again=again)
-            write_to_group(neuron_group, 'spike_indices', t, again)
+            write_to_group(neuron_group, 'spike_indices', t, change_spike_train)
         else:
             t = neuron_group['spike_indices']
 
-        # usable spikes mask
-        if 'sane_spikes' not in neuron_group or again:
-            if sane_spikes is None and\
-                sane_periods is not None:
-                sane_periods = check_periods(sane_periods)
-                sane_periods*=samp_rate
-                sane_spikes = (t*0).astype(bool)
-                for sane_period in sane_periods:
-                    sane_spikes = sane_spikes|(t>=sane_period[0])&(t<=sane_period[1])
-            elif sane_spikes is None and\
-                optostims is not None and\
-                sane_before_opto:
-                # Only consider spikes 10s before first opto onset
-                sane_spikes = (t < (optostims[0,0]-10)*samp_rate)
-            else:
-                sane_spikes = (t*0+1).astype(bool)
-            write_to_group(neuron_group, 'sane_spikes', sane_spikes, again)
-
         # optostims
-        if 'optostims' not in neuron_group or again:
+        change_optostims = overwrite_h5 or ("optostims" in selective_overwrite)
+        if 'optostims' not in neuron_group or change_optostims:
             if optostims is None and optostims_from_sync:
                 ons, offs = get_npix_sync(dp, verbose=False)
                 if sync_chan_id is None:
@@ -274,11 +267,32 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
                 optostims = optostims[opto_m,:]
 
             if optostims is not None:
-                write_to_group(neuron_group, 'optostims', optostims, again)
+                write_to_group(neuron_group, 'optostims', optostims, change_optostims)
+    
+        # usable spikes mask
+        change_mask = overwrite_h5 or ("sane_spikes" in selective_overwrite)
+        if 'sane_spikes' not in neuron_group or change_mask:
+            if sane_spikes is None and\
+                sane_periods is not None:
+                sane_periods = check_periods(sane_periods)
+                sane_periods*=samp_rate
+                sane_spikes = (t*0).astype(bool)
+                for sane_period in sane_periods:
+                    sane_spikes = sane_spikes|(t>=sane_period[0])&(t<=sane_period[1])
+            elif sane_spikes is None and\
+                optostims is not None and\
+                sane_before_opto:
+                # Only consider spikes 10s before first opto onset
+                sane_spikes = (t < (optostims[0,0]-10)*samp_rate)
+            else:
+                sane_spikes = (t*0+1).astype(bool)
+            write_to_group(neuron_group, 'sane_spikes', sane_spikes, change_mask)
+
 
         # waveforms
         k = ['mean_waveform_preprocessed', 'amplitudes', 'voltage_sample', 'peakchan_SNR']
-        if not all_keys_in_group(k, neuron_group) or again:
+        change_waveforms = overwrite_h5 or any([key in selective_overwrite for key in k])
+        if not all_keys_in_group(k, neuron_group) or change_waveforms:
             # must recompute chan_bottom and chan_top - suboptimal, can be rewritten
             dsm_tuple = wvf_dsmatch(dp, unit, t_waveforms=waveform_samples, periods=periods,
                                     again=again_wvf, plot_debug=plot_debug, verbose=verbose,
@@ -292,19 +306,20 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             peak_chan_rel = np.nonzero(peak_chan == chan_range[chan_range_m])[0]
             dsm_waveform_chunk = dsm_waveform[:, chan_bottom:chan_top]
             write_to_group(neuron_group, 'mean_waveform_preprocessed',
-                           dsm_waveform_chunk.T, again)
+                           dsm_waveform_chunk.T, change_waveforms)
             write_to_group(neuron_group, 'consensus_waveform',
-                           dsm_waveform_chunk.T*np.nan, again)
+                           dsm_waveform_chunk.T*np.nan, change_waveforms)
             cm = chan_map(dp)
             write_to_group(neuron_group, 'channel_ids',
                            np.arange(chan_bottom, chan_top, dtype=np.dtype('uint16')),
-                           again)
-            write_to_group(neuron_group, 'channelmap', cm[chan_bottom:chan_top, 1:], again)
+                           change_waveforms)
+            write_to_group(neuron_group, 'channelmap', cm[chan_bottom:chan_top, 1:2], change_waveforms)
 
         # Extract voltage snippets
         k = ['amplitudes', 'channel_noise_std', 'peakchan_SNR', 'voltage_sample']
+        change_snippet = overwrite_h5 or any([key in selective_overwrite for key in k])
         if not all_keys_in_group(k, neuron_group)\
-            or again:
+            or change_snippet:
             if center_raw_window_on_spikes:
                 t = h5_file[relative_unit_path+'/spike_indices'][...]/samp_rate
                 if raw_window[1]>t[0]: # spike starting after end of original window
@@ -316,7 +331,8 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
 
         # quality metrics
         k = ['amplitudes', 'channel_noise_std', 'peakchan_SNR']
-        if not all_keys_in_group(k, neuron_group) or again:
+        change_metrics = overwrite_h5 or any([key in selective_overwrite for key in k])
+        if not all_keys_in_group(k, neuron_group) or change_metrics:
                 
             amps = np.load(dp/'amplitudes.npy').squeeze()[ids(dp, unit, periods=periods)]
             
@@ -326,11 +342,13 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             peakchan_N = std_estimate[peak_chan_rel]
             peakchan_SNR = peakchan_S / peakchan_N
             
-            write_to_group(neuron_group, 'amplitudes', amps, again)
-            write_to_group(neuron_group, 'channel_noise_std', std_estimate, again)
-            write_to_group(neuron_group, 'peakchan_SNR', peakchan_SNR, again)
-    
-        if ('fn_fp_filtered_spikes' not in neuron_group or again) and include_fp_fn_mask:
+            write_to_group(neuron_group, 'amplitudes', amps, change_metrics)
+            write_to_group(neuron_group, 'channel_noise_std', std_estimate, change_metrics)
+            write_to_group(neuron_group, 'peakchan_SNR', peakchan_SNR, change_metrics)
+
+        
+        change_fn_fp = overwrite_h5 or ('fn_fp_filtered_spikes' in selective_overwrite)
+        if ('fn_fp_filtered_spikes' not in neuron_group or overwrite_h5) and include_fp_fn_mask or change_fn_fp:
             # get good spikes mask for all spikes
             # because trn_filtered can only work on a contiguous chunk
             if isinstance(periods, str): # can only be 'all', given check_periods
@@ -348,10 +366,11 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
                 periods_mask = np.isin(t_all, t)
                 fp_fn_good_spikes = fp_fn_good_spikes[periods_mask]
 
-            write_to_group(neuron_group, 'fn_fp_filtered_spikes', fp_fn_good_spikes, again)
+            write_to_group(neuron_group, 'fn_fp_filtered_spikes', fp_fn_good_spikes, change_fn_fp)
             
         # voltage snippets
-        if ('voltage_sample' not in neuron_group or again)\
+        change_voltage_snippets = overwrite_h5 or ('voltage_snippets' in selective_overwrite)
+        if ('voltage_sample' not in neuron_group or change_voltage_snippets)\
             and include_raw_snippets:
             # Only store the voltage sample for the primary channel
             peak_chan = neuron_group['primary_channel']
@@ -359,11 +378,11 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             c1 = max(0,int(chunk.shape[0]/2-raw_snippet_halfrange))
             c2 = min(chunk.shape[0]-1, int(chunk.shape[0]/2+raw_snippet_halfrange+1))
             raw_snippet = chunk[c1:c2,:]
-            write_to_group(neuron_group, 'voltage_sample', raw_snippet) # still centered on peak channel, but half the size
-            write_to_group(neuron_group, 'voltage_sample_start_index', int(raw_window[0] * samp_rate))
-            write_to_group(neuron_group, 'scaling_factor', meta['bit_uV_conv_factor'])
+            write_to_group(neuron_group, 'voltage_sample', raw_snippet, change_voltage_snippets) # still centered on peak channel, but half the size
+            write_to_group(neuron_group, 'voltage_sample_start_index', int(raw_window[0] * samp_rate), change_voltage_snippets)
+            write_to_group(neuron_group, 'scaling_factor', meta['bit_uV_conv_factor'], change_voltage_snippets)
             
-        if ('whitened_voltage_sample' not in neuron_group or again)\
+        if ('whitened_voltage_sample' not in neuron_group or change_voltage_snippets)\
             and include_whitened_snippets:
             if center_raw_window_on_spikes:
                 t = h5_file[relative_unit_path+'/spike_indices'][...]/samp_rate
@@ -387,20 +406,20 @@ def add_unit_h5(h5_path, dp, unit, lab_id, periods='all',
             c1 = max(0,int(white_chunk.shape[0]/2-raw_snippet_halfrange))
             c2 = min(white_chunk.shape[0]-1, int(white_chunk.shape[0]/2+raw_snippet_halfrange+1))
             raw_snippet = white_chunk[c1:c2,:].astype(np.float32)
-            write_to_group(neuron_group, 'whitened_voltage_sample', raw_snippet)
+            write_to_group(neuron_group, 'whitened_voltage_sample', raw_snippet, change_voltage_snippets)
 
         # layer
-        write_to_group(neuron_group, 'phyllum_layer', 0, again)
-        write_to_group(neuron_group, 'human_layer', 0, again)
+        write_to_group(neuron_group, 'phyllum_layer', 0, overwrite_h5)
+        write_to_group(neuron_group, 'human_layer', 0, overwrite_h5)
 
         # ground truth labels
-        write_to_group(neuron_group, 'expert_label', 0, again)
-        write_to_group(neuron_group, 'optotagged_label', 0, again)
+        write_to_group(neuron_group, 'expert_label', 0, overwrite_h5)
+        write_to_group(neuron_group, 'optotagged_label', 0, overwrite_h5)
 
         # predicted labels
-        write_to_group(neuron_group, 'lisberger_label', 0, again)
-        write_to_group(neuron_group, 'hausser_label', 0, again)
-        write_to_group(neuron_group, 'medina_label', 0, again)
+        write_to_group(neuron_group, 'lisberger_label', 0, overwrite_h5)
+        write_to_group(neuron_group, 'hausser_label', 0, overwrite_h5)
+        write_to_group(neuron_group, 'medina_label', 0, overwrite_h5)
 
     return relative_unit_path
 
@@ -457,7 +476,7 @@ def add_json_datasets_to_h5(json_path, h5_path, lab_id, preprocess_if_raw=False,
         units=ds['units']
         ss=ds['ss']
         cs=ds['cs']
-        good_units = list(get_units(dp, 'good'))
+        good_units = list(get_units(dp, 'good', again=again))
         sane_times = ds["sane_times"] if "sane_times" in ds else None
 
         if include_all_good:
@@ -482,12 +501,13 @@ def add_json_datasets_to_h5(json_path, h5_path, lab_id, preprocess_if_raw=False,
 
 def add_json_datasets_to_h5_hausser(json_path, h5_path, again=False, include_raw_snippets=False,
                                     delete_original_data=False, data_deletion_double_check=False,
-                                    include_all_good=False):
+                                    include_all_good=False, overwrite_h5=False, **kwargs):
 
     add_json_datasets_to_h5(json_path, h5_path, "hausser", preprocess_if_raw=False,
                             delete_original_data=delete_original_data, data_deletion_double_check=data_deletion_double_check,
                             again=again, include_raw_snippets=include_raw_snippets, verbose=False,
-                            optostims_from_sync=True, optostims_threshold=20*60, include_all_good=include_all_good)
+                            optostims_from_sync=True, optostims_threshold=20*60, sane_before_opto=True,
+                            include_all_good=include_all_good, overwrite_h5=overwrite_h5, **kwargs)
 
 
 def add_data_to_unit_h5(h5_path, dataset, unit, data, field):
